"""
Constants for Postman receiver.
"""
BOOTSTRAP_PATH = "/sdk/bootstrap"
TRACE_RECEIVER_PATH = "/traces"
HEALTHCHECK_PATH = "/sdk/health"

DEFAULT_HEALTH_PING_INTERVAL_MILLISECONDS = 60000
DEFAULT_HEALTH_PING_INTERVAL_SECONDS = DEFAULT_HEALTH_PING_INTERVAL_MILLISECONDS // 1000

EXPONENTIAL_BACKOFF_BASE = 2
BOOTSTRAP_RETRY_COUNT = 2
